/***************************************************************
	mwpp - Converts .xlabcesam files to .window files

	1.00 85-11-28 ldab/hn: Works (hopefully...)
	by Henrik Norbeck, Luxor Datorer AB 

	strings.c Contains string printing and string handling functions

	find		Finds a menu in the menu table
	pr_pstring	Splits the menu text and prints the pstring
	ps_lstring	Prints the lstring
	search		Searches for a character in a command string
	uppercase	Converts a string to upper case
	noescquot	Removes quotes around a string and escapes in it.
***************************************************************/

/* Header files */
#include <stdio.h>
#include "mwpp.h"

extern FILE *wd;

/*	string arrays for the menus */
extern char nbuf[13][20];
extern char mbuf[13][64];
extern char pbuf[13][64];

extern struct menu_t menu[];
extern int n_menus;

extern char *menu_str[13][4];
extern int n_mstr[13];

/***	Find a menu in the menu table ***/
find(name)
char *name;
{
    register int i;

    putchar('\n');
    for (i=0; i<n_menus; i++) {
	printf("%s %s\n", name, menu[i].name);
	if (strcmp(name, menu[i].name) == 0)
	    return(i);
	}
    return (ERROR);
}

/***	Split menu text and print pstring ***/
pr_pstring(entries, current)
int entries, current;
{
    register int e, i, j, old_j;
    int l, pxor, pyor;

    fprintf(wd, "pstring%d:\\\n", current+1);
    fprintf(wd, "\\e:0;0m\\e:767;0;;1d\\e:0;60m\\e:767;60;;1d\\\n");
    fprintf(wd, "\\e:0;120m\\e:767;120;;1d\\e:0;0m\\e:0;120;;1d\\\n");
    fprintf(wd, "\\e:128;0m\\e:128;120;;1d\\e:256;0m\\e:256;120;;1d\\\n");
    fprintf(wd, "\\e:384;0m\\e:384;120;;1d\\e:512;0m\\e:512;120;;1d\\\n");
    fprintf(wd, "\\e:640;0m\\e:640;120;;1d\\e:767;0m\\e:767;120;;1d\\\n");

    for (e=1; e<entries; e++) {
	/* Split text so that it will fit into the icon */
	menu_str[e][0] = mbuf[e];
	while (*menu_str[e][0] == ' ')		/* Remove leading blanks */
	    menu_str[e][0]++;
	i = 0;
	while ((l=strlen(menu_str[e][i]))>13 && i<3) {
	    while (*menu_str[e][i] == ' ')	/* Remove blanks */
		menu_str[e][i]++;
	    menu_str[e][++i] = menu_str[e][i];
	    old_j = j = 0;
	    while (j < 14) {
		while (menu_str[e][i][j]!=' ' && menu_str[e][i][j])
		    j++;
		if (j < 14)
		    old_j = j;
		while (menu_str[e][i][j]==' ')
		    j++;
	        }
	    if (NOT old_j) {/* Too long word. Cut it off! */
		old_j = 13;
		for (j=l; j>=old_j; j--)
		    menu_str[e][i][j+1] = menu_str[e][i][j];
		}
	    menu_str[e][i][old_j] = '\0';
	    menu_str[e][i] += old_j+1;
	    }
	if (i==4)	/* Text too long. Cut it off! */
	    menu_str[e][3][13] = '\0';
	else
	    i++;	/* No of rows */
	n_mstr[e] = i;

	/* Calculate x and y for the icon */
	if (e == entries-1) {
	    pxor = 641;
	    pyor = 1;
	    }
	else {
	    pxor = (e>6 ? e-6 : e)*128 - 127;
	    pyor = (e>6 ? 1 : 61);
	    }

	/* Print text for this icon */
	for (j=0; j<i; j++)
	    fprintf(wd, "\\e:%d;%dm\\e:1H%s",
		    pxor+63-9*strlen(menu_str[e][j])/2,
		    pyor + 15 + i*7 - j*14,
		    menu_str[e][j]);
	if (e != entries-1)
	    putc('\\', wd);
	putc('\n', wd);
	}
    putc('\n', wd);
}

/***	Print lstring ***/
pr_lstring(entries, current)
int entries, current;
{
    register int e, j;
    int lxor, lyor;

    fprintf(wd, "lstring%d:\\\n", current+1);
    fprintf(wd, "\\e:0;0m\\e:0;720;;1d\\e:127;0m\\e:127;720;;1d\\\n");
    fprintf(wd, "\\e:0;720m\\e:126;720;;1d\\e:0;660m\\e:126;660;;1d\\\n");
    fprintf(wd, "\\e:0;600m\\e:126;600;;1d\\e:0;540m\\e:126;540;;1d\\\n");
    fprintf(wd, "\\e:0;480m\\e:126;480;;1d\\e:0;420m\\e:126;420;;1d\\\n");
    fprintf(wd, "\\e:0;360m\\e:126;360;;1d\\e:0;300m\\e:126;300;;1d\\\n");
    fprintf(wd, "\\e:0;240m\\e:126;240;;1d\\e:0;180m\\e:126;180;;1d\\\n");
    fprintf(wd, "\\e:0;120m\\e:126;120;;1d\\e:0;60m\\e:126;60;;1d\\\n");
    fprintf(wd, "\\e:0;0m\\e:126;0;;1d\\\n");

    for (e=1; e<entries; e++) {
	/* The strings were already split in pr_pstring */

	/* Calculate x and y for the icon */
	lxor = 1;
	if (e == entries-1)
	    lyor = 1;
	else
	    lyor = 721 - e*60;

	/* Print text for this icon */
	for (j=0; j<n_mstr[e]; j++)
	    fprintf(wd, "\\e:%d;%dm\\e:1H%s",
		    lxor+62-9*strlen(menu_str[e][j])/2,
		    lyor + 15 + n_mstr[e]*7 - j*14,
		    menu_str[e][j]);
	if (e != entries-1)
	    putc('\\', wd);
	putc('\n', wd);
	}
    putc('\n', wd);
}

/***	Search for a character in a string, skipping escaped characters
	and strings delimited by " or ' ***/
char *search(s, match)
register char *s, match;
{
    register char c;

    while (c = *s) {
	if (c == match)
	    return (s);
	s++;
	if (c == '\\')	/* Skip escaped characters */
	    if (NOT *s)
		return (NULL);
	    else
		s++;
	else if (c=='"' || c=='\'')	/* Skip strings delimited by " or ' */
	    for (; *s && *s!=c; s++)
		if (*s == '\\')		/* Skip escaped character */
		    if (NOT *(++s))
			return (NULL);
	}
    return (NULL);
}

/***	Take away escapes from a string and surrounding quotes ***/
char *noescquot(s)
register char *s;
{
    char *noescape();

    if (*s == '"' || *s == '\'') {
	strcpy(s, s+1);
	s[strlen(s)-1] = '\0';
	}
    return (noescape(s));
}
